/*!\file SystemMatricesx
 * \brief: create system matrices (stiffness matrix, loads vector)
 */

#include "./SystemMatricesx.h"

#include "../shared/shared.h"
#include "../include/include.h"
#include "../toolkits/toolkits.h"
#include "../EnumDefinitions/EnumDefinitions.h"

void SystemMatricesx(Mat* pKgg, Vec* ppg,DataSet* elements,DataSet* nodes, DataSet* vertices,DataSet* loads,DataSet* materials, Parameters* parameters,
		int kflag,int pflag,int analysis_type,int sub_analysis_type){
	
	extern int num_procs;
	extern int my_rank;
	
	/*intermediary: */
	int gsize;
	int connectivity;
	int numberofdofspernode;
	
	/*output: */
	Mat Kgg=NULL;
	Vec pg=NULL;

	/*First, get elements and loads configured: */
	elements->Configure(elements,loads, nodes,vertices, materials,parameters);
	nodes->Configure(elements,loads, nodes,vertices, materials,parameters);
	loads->Configure(elements, loads, nodes,vertices, materials,parameters);
	parameters->Configure(elements,loads, nodes,vertices, materials,parameters);

	/*Recover parameters: */
	parameters->FindParam(&connectivity,ConnectivityEnum);
	parameters->FindParam(&numberofdofspernode,NumberOfDofsPerNodeEnum);

	/*Get size of matrix: */
	gsize=nodes->NumberOfDofs();

	/*Allocate Kgg and pg: */
	if(kflag)Kgg=NewMat(gsize,gsize,NULL,&connectivity,&numberofdofspernode);
	if(pflag)pg=NewVec(gsize);

	/*Fill stiffness matrix and right hand side vector, from elements: */
	if(kflag)elements->CreateKMatrix(Kgg,analysis_type,sub_analysis_type);
	if(pflag)elements->CreatePVector(pg,analysis_type,sub_analysis_type);
	
	/*Fill stiffness matrix and right hand side vector, from loads: */
	if(kflag)loads->CreateKMatrix(Kgg,analysis_type,sub_analysis_type);
	if(pflag)loads->CreatePVector(pg,analysis_type,sub_analysis_type);

	/*Assemble matrices: */
	if(kflag){
		MatAssemblyBegin(Kgg,MAT_FINAL_ASSEMBLY);
		MatAssemblyEnd(Kgg,MAT_FINAL_ASSEMBLY);
		MatCompress(Kgg);
	}
	if(pflag){
		VecAssemblyBegin(pg);
		VecAssemblyEnd(pg);
	}
	
	/*Assign output pointers: */
	*pKgg=Kgg;
	*ppg=pg;
}
